/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.tree;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.AbstractAction;
import nl.cloudfarming.client.fleet.machine.windows.ListOverviewTopComponent;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.Machine.MachineMetaInfo;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import org.netbeans.api.project.Project;
import org.openide.filesystems.*;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Wytse Visser
 */
public abstract class MachineCategoryNode extends FilterNode {

    private Project project;

    public MachineCategoryNode(Node original, Children children, Project project, Lookup lookup) {
        super(original, children, createLookup(original, lookup));
        this.project = project;
        setDisplayName(getMetaInfo().getDisplayName());
    }

    private static Lookup createLookup(Node original, Lookup lookup) {
        String name = lookup.lookup(MachineMetaInfo.class).getDisplayName();

        return new ProxyLookup(original.getLookup(), lookup, Lookups.fixed(new SimpleLayer(name)));
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(getMetaInfo().getSmallIconResource());
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    public final MachineMetaInfo getMetaInfo() {
        return getLookup().lookup(MachineMetaInfo.class);
    }
    
    protected Project getProject() {
        return project;
    }

    @NbBundle.Messages({"fleet.machine.category_node_open_action_name=Open"})
    protected class OpenAction extends AbstractAction {

        public OpenAction() {
            super(Bundle.fleet_machine_category_node_open_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            ListOverviewTopComponent overviewComponent = ListOverviewTopComponent.findInstance(MachineCategoryNode.this);
            overviewComponent.open();
            overviewComponent.requestActive();
        }

        @Override
        public boolean isEnabled() {
            return MachineCategoryNode.this.getChildren().getNodesCount(true) > 0;
        }
    }

}