/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.tree;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.swing.Action;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.Machine.MachineMetaInfo;
import org.openide.actions.OpenAction;
import org.openide.filesystems.FileObject;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.nodes.Sheet;
import org.openide.util.ImageUtilities;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.PasteType;

/**
 *
 * @author Wytse Visser && Maciek Dulkiewicz
 */
public abstract class MachineNode extends FilterNode {

    private Machine machine;
    private Node original;

    public MachineNode(Node node) {
        super(node, Children.LEAF, node.getLookup());
        original = node;
        original.setDisplayName(getMachine().getName());
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(getMetaInfo().getSmallIconResource());
    }

    @Override
    public String getName() {
        return getMachine().getName();
    }

    @Override
    public String getDisplayName() {
        return getName();
    }

    @Override
    public void setName(String name) {
        String oldName = getName();
        getMachine().setName(name);
        original.setDisplayName(name);
        fireDisplayNameChange(oldName, name);
    }

    @Override
    public void destroy() throws IOException {
        getMachineFile().delete();
    }

    protected FileObject getMachineFile() {
        return (FileObject) getLookup().lookup(FileObject.class);
    }

    @Override
    public boolean canDestroy() {
        return true;
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public Action getPreferredAction() {
        return SystemAction.get(OpenAction.class);
    }

    @Override
    public Action[] getActions(boolean context) {
        return super.getActions(context);
    }

    protected Sheet createSheet() {
        return getMachine().createSheet();
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
    }

    @Override
    public Transferable clipboardCut() throws IOException {
        return clipboardCopy();
    }

    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    protected final Machine getMachine() {
        if (machine == null) {
            machine = getLookup().lookup(Machine.class);
            machine.addPropertyChangeListener(new AutoSaver());
        }
        return machine;
    }

    protected abstract MachineMetaInfo getMetaInfo();

    protected abstract void saveMachine();

    private class AutoSaver implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    if (getMachineFile().canWrite()) {
                        saveMachine();
                    }
                }
            }).start();
        }
    }
}
