/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.windows;

import java.util.List;
import javax.swing.GroupLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.DefaultTableModel;
import nl.cloudfarming.client.fleet.model.Machine;
import org.jdesktop.swingx.JXTable;
import org.openide.nodes.Sheet;
import org.openide.windows.WindowManager;

/**
 * This panel contains a table that shows a list of machines with their
 * properties.
 *
 * @author Wytse Visser
 */
public class ListOverviewPanel extends JPanel {

//    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.machine.windows.ListOverviewPanel");
    private JScrollPane scrollPane;
    private JXTable machineTable;

    public ListOverviewPanel() {
        initComponents();
    }

    private void initComponents() {
        machineTable = new JXTable(new MachineListModel());
        scrollPane = new JScrollPane();
        scrollPane.setViewportView(machineTable);
        scrollPane.setPreferredSize(getMaximumSize());

        setBackground(java.awt.Color.white);

        GroupLayout layout = new GroupLayout(this);
        this.setLayout(layout);

        layout.setHorizontalGroup(
                layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
                layout.createSequentialGroup().addComponent(scrollPane)));

        layout.setVerticalGroup(
                layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
                layout.createSequentialGroup().addComponent(scrollPane)));
    }

    /**
     * Updates the list of machines displayed in this panel with the given one.
     *
     * @param machines The machines to show
     */
    public void setMachines(List<Machine> machines) {
        MachineListModel model = (MachineListModel) machineTable.getModel();
        model.setMachines(machines);
    }

    /**
     * The table model for machines.
     */
    private static class MachineListModel extends DefaultTableModel {

        private static final String[] COLUMN_NAMES = new String[]{
            "Name",
            "Type",
            "Brand",
            "Model",
            "Year",
            "Purchase date",
            "Purchase price",
            "Mileage",};

        public MachineListModel() {
            initColumns();
        }

        private void initColumns() {
            for (String columnName : COLUMN_NAMES) {
                addColumn(columnName);
            }
        }

        /**
         * Updates the machines to the given machines. To prevent concurrency
         * problems, this method is performed when the UI is ready.
         *
         * @param machines The new list of machines
         */
        public void setMachines(final List<Machine> machines) {
            WindowManager.getDefault().invokeWhenUIReady(
                    new Runnable() {

                        @Override
                        public void run() {
                            //calling removeRow(0) in a loop to clean model is slow.
                        	//remaining items need to be shifted, listeners are fired etc.
                        	//let model do it
                        	setRowCount(0);
                        	                            
                            for (Machine machine : machines) {
                                addRow(new Object[]{
                                            machine.getName(),
                                            machine.getType(),
                                            machine.getBrand(),
                                            machine.getModelNumber(),
                                            machine.getYear(),
                                            machine.getPurchaseDate(),
                                            machine.getPurchasePrice(),
                                            machine.getMileage()
                                        });
                            }
                        }
                    });
        }
    }
}
