/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.logging.Logger;
import javax.swing.JLabel;
import javax.swing.JPanel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.fleet.model.MachineProperty;
import org.openide.explorer.propertysheet.PropertyPanel;
import org.openide.nodes.Node.Property;

/**
 * This class represents a visual panel for a dynamic property form. It shows a
 * form with fields for its properties.
 *
 * @author Wytse Visser
 */
class MachineDynamicVisualPanel extends JPanel {

    /**
     * Creates new form MachineDynamicVisualPanel
     */
    public MachineDynamicVisualPanel() {
        setLayout(new MigLayout("wrap 2"));
    }

    /**
     * Add a new property field to this visual panel. The PropertyPanel class is
     * used to create this field, so the property editor is based on the
     * property type.
     *
     * @param property The property
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    public void addPropertyField(MachineProperty property) throws IllegalAccessException, InvocationTargetException {
        Property realProperty = property.getProperty();
        add(new JLabel(realProperty.getName()));
        add(new PropertyPanel(realProperty), "width min:300:max");
    }
}
