/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;
import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import net.miginfocom.swing.MigLayout;
import org.jdesktop.swingx.JXDatePicker;
import org.openide.util.NbBundle;

/**
 * The visual representation of the first machine creation wizard panel.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({"generalPropertiesStepName=General details"})
public final class MachineVisualStartPanel extends JPanel implements DocumentListener, ActionListener {

//    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.vehicle.wizard.MachineVisualStartPanel");
    private MachineWizardStartPanel wizardPanel;
    private JComboBox brandField;
    private JLabel brandLabel;
    private JTextField customBrandField;
    private JLabel customBrandLabel;
    private JTextField customModelNumberField;
    private JLabel customModelNumberLabel;
    private JSpinner mileageField;
    private JLabel mileageLabel;
    private JComboBox modelField;
    private JLabel modelLabel;
    private JTextField nameField;
    private JLabel nameLabel;
    private JXDatePicker purchaseDateField;
    private JLabel purchaseDateLabel;
    private JSpinner purchasePriceField;
    private JLabel purchasePriceLabel;
    private JComboBox typeField;
    private JLabel typeLabel;
    private JSpinner yearField;
    private JLabel yearLabel;

    /**
     * Creates new form MachineVisualStartPanel
     */
    public MachineVisualStartPanel(MachineWizardStartPanel wizardPanel) {
        this.wizardPanel = wizardPanel;
        setLayout(new MigLayout("wrap 2, hidemode 3"));
        initComponents();
    }

    @Override
    public String getName() {
        return Bundle.generalPropertiesStepName();
    }

    public String getNameValue() {
        return nameField.getText().trim();
    }

    public void setNameValue(String name) {
        nameField.setText(name);
    }

    public String getBrandValue() {
        Object selectedItem = brandField.getSelectedItem();
        if (selectedItem == null || selectedItem.equals("")) {
            return null;
        }
        return brandField.getSelectedItem().toString();
    }

    public int getMileageValue() {
        return Integer.parseInt(mileageField.getValue().toString());
    }

    public String getModelValue() {
        Object selectedItem = modelField.getSelectedItem();
        if (selectedItem == null || selectedItem.equals("")) {
            return null;
        }
        return modelField.getSelectedItem().toString();
    }

    public Date getPurchaseDateValue() {
        return purchaseDateField.getDate();
    }

    public double getPurchasePriceValue() {
        return Integer.parseInt(purchasePriceField.getValue().toString());
    }

    public String getTypeValue() {
        Object selectedItem = typeField.getSelectedItem();
        if (selectedItem == null || selectedItem.equals("")) {
            return null;
        }
        return typeField.getSelectedItem().toString();
    }

    public int getYearValue() {
        return Integer.parseInt(yearField.getValue().toString());
    }

    @Override
    public void insertUpdate(DocumentEvent e) {
        wizardPanel.fireChangeEvent();
    }

    @Override
    public void removeUpdate(DocumentEvent e) {
        wizardPanel.fireChangeEvent();
    }

    @Override
    public void changedUpdate(DocumentEvent e) {
        wizardPanel.fireChangeEvent();
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        wizardPanel.fireChangeEvent();
    }

    void setTypes(Object[] types) {
        setComboBoxItems(typeField, types);
    }

    void setBrands(Object[] brands) {
        setComboBoxItems(brandField, brands);
    }

    void setModels(Object[] models) {
        setComboBoxItems(modelField, models);
    }

    void setYear(int year) {
        yearField.setValue(year);
    }

    void enableCustomBrandField(boolean enable) {
        customBrandField.setVisible(enable);
        customBrandLabel.setVisible(enable);
    }

    void enableCustomModelNumberField(boolean enable) {
        customModelNumberField.setVisible(enable);
        customModelNumberLabel.setVisible(enable);
    }

    String getCustomModelValue() {
        return customModelNumberField.getText();
    }

    String getCustomBrandValue() {
        return customBrandField.getText();
    }

    private void setComboBoxItems(JComboBox comboBox, Object[] items) {
        Object previousSelection = comboBox.getSelectedItem();
        comboBox.setModel(new DefaultComboBoxModel(items));
        if (items.length == 1) { // Just one item selectable, select it
            comboBox.setSelectedIndex(0);
        } else if (previousSelection != null) {
            comboBox.setSelectedItem(previousSelection);
        }
    }

    private void initComponents() {
        typeLabel = new JLabel("Type");
        typeField = new JComboBox();
        typeField.addActionListener(this);
        add(typeLabel);
        add(typeField);

        brandLabel = new JLabel("Brand");
        brandField = new JComboBox();
        brandField.addActionListener(this);
        add(brandLabel);
        add(brandField);

        customBrandLabel = new JLabel("Custom brand");
        customBrandField = new JTextField(20);
        customBrandField.getDocument().addDocumentListener(this);
        add(customBrandLabel);
        add(customBrandField);

        modelLabel = new JLabel("Model number");
        modelField = new JComboBox();
        modelField.addActionListener(this);
        add(modelLabel);
        add(modelField);

        customModelNumberLabel = new JLabel("Custom model");
        customModelNumberField = new JTextField(20);
        customModelNumberField.getDocument().addDocumentListener(this);
        add(customModelNumberLabel);
        add(customModelNumberField);

        nameLabel = new JLabel("Name");
        nameField = new JTextField(20);
        nameField.getDocument().addDocumentListener(this);
        add(nameLabel);
        add(nameField);

        yearLabel = new JLabel("Year");
        yearField = new JSpinner(createYearSpinnerModel());
        yearField.setEditor(new JSpinner.NumberEditor(yearField, "0000"));
        add(yearLabel);
        add(yearField);

        purchaseDateLabel = new JLabel("Purchase date");
        purchaseDateField = new JXDatePicker(Calendar.getInstance().getTime());
        add(purchaseDateLabel);
        add(purchaseDateField);

        purchasePriceLabel = new JLabel("Purchase price");
        purchasePriceField = new NonNegativeNumberSpinner(100);
        add(purchasePriceLabel);
        add(purchasePriceField);

        mileageLabel = new JLabel("Mileage");
        mileageField = new NonNegativeNumberSpinner(100);
        add(mileageLabel);
        add(mileageField);
    }

    private static SpinnerModel createYearSpinnerModel() {
        final int currentYear = Calendar.getInstance().get(Calendar.YEAR);
        return new SpinnerNumberModel(currentYear, currentYear - 100, currentYear, 1);
    }

    private class NonNegativeNumberSpinner extends JSpinner {

        private int width;

        public NonNegativeNumberSpinner(int width) {
            super(new SpinnerNumberModel(0, 0, null, 1));
            setEditor(new NumberEditor(this));
            this.width = width;
        }

        @Override
        public Dimension getPreferredSize() {
            Dimension d = super.getPreferredSize();
            return new Dimension(width, d.height);
        }
    }
}
