/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.windows.WindowManager;

/**
 *
 * @author Wytse Visser
 */
public abstract class MachineWizardPanel implements WizardDescriptor.FinishablePanel<WizardDescriptor> {

    private JComponent component;

    @Override
    public JComponent getComponent() {
        if (component == null) {
            component = createComponent();
        }
        return component;
    }

    protected JComponent createComponent() {
        return new WizardComponent();
    }

    @Override
    public HelpCtx getHelp() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public boolean isFinishPanel() {
        return isValid();
    }
    
    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }
    
    protected static ImageIcon getIcon() {
        return ImageUtilities.loadImageIcon("nl/cloudfarming/client/fleet/vehicle/steering_wheel_332.png", true);
    }

    protected void setTitle(final String title) {
        WindowManager.getDefault().invokeWhenUIReady(
                new Runnable() {

                    @Override
                    public void run() {
                        ((WizardComponent) getComponent()).setTitle(title);
                    }
                });
    }

    protected abstract JPanel getVisual();
    
    @Override
    public abstract void readSettings(WizardDescriptor wiz);

    @Override
    public abstract void storeSettings(WizardDescriptor wiz);

    private class WizardComponent extends JScrollPane {

        private ASBodyPanel bodyPanel;

        public WizardComponent() {
            bodyPanel = new ASBodyPanel(getVisual());
            bodyPanel.setIcon(getIcon());
            super.setViewportView(bodyPanel);
        }

        public void setTitle(String title) {
            bodyPanel.setTitle(title);
        }

        @Override
        public String getName() {
            return getVisual().getName();
        }
    }
}